# R-code to fit the univariate latent regression model:
#    y = beta0 + beta1*x + epsilon
# where x is a latent predictor with a beta distribution,
# and epsilon is a N(0,sigma^2) random error.

source("emnorm.r")  # Load code to fit a finite mixture model
y <-  read.table("HamDdifferencesWeek1.dat",header=F)
y <- as.matrix(y[,2])

hist(y,20, freq=FALSE, xlab="Difference", main="Prozac Difference Data",
      ylab="Relative Frequency")

n<- dim(y)[1]
t <- seq(0,1, by = .005)
shapiro.test(y)

# Set initial values for algorithm and define inputs to algorithm

k  <-  2                      # k = number of mixture components
prior  <-  matrix(1,k)/k      # Define a vector of initial values for the prior probabilities
pool   <-  0                  #  indicator for equality of variances: pool=0: unequal variances; pool=1: equal variances
S <- cov(y)
p <- dim(y)[2]
# Run k-means to get initial values for mixture component means
kmean  <-  kmeans(y, k, iter.max = 100)
mu <- as.matrix(sort(kmean$center))      # Initial component means as a k x p matrix
var <-  array(S, c(p,p, k))   # Initial covariance matrix estimates
results <- emnorm(k,y, prior, mu, var, pool)
muvar<- t(matrix(cbind(results$prior,results$mu,results$var),6,1))


s <- sqrt(sum(kmean$withinss)/n)  # initial value of sigma
bhat <- matrix(1,2,1)
bhat[1,1] <- results$mu[1]  # Initial values for beta0 and beta1 estimates
bhat[2,1] <- -results$mu[1]+results$mu[2]
ab <- matrix(1,2,1) # give initial values for beta parameters


# starting values for Prozac improvement data:
#ab[1,1] <- 0.897
#ab[2,1] = 4.313
#bhat[1,1]=-0.629
#bhat[2,1]= 30.849
#s <- 2.278

abold <- ab
bhatold <- bhat 
sold <- s


error <- 1/10^7  # convergence criterion - iterate until the squared differences
                 # between parameter estimates on 2 consecutive iterations is less
                 # than error
diff <- 10^8
nit <- 1
maxiter <- 1000   # Maximum number of iterations
while ( diff > error & nit <= maxiter)            # start iterations 
{
nit <- nit+1

# E-step: Compute E[x_i |y_i, theta], theta = current parameter values
# E-step: Compute E[x_i^2 |y_i, theta], theta = current parameter values
# E-step: Compute E[ln(x_i) |y_i, theta], theta = current parameter values
# E-step: Compute E[ln(1-x_i) |y_i, theta], theta = current parameter values


xx <- matrix(0,n,4) # matrix holding E[x1^2|y],E[x2^2|y], and
                    # E[ln(x)|y], E[ln(1-x)|y] 
for(i in 1:n)
{
yi = y[i,1]

fd<- function(x) {dnorm(yi, mean=(bhat[1,1] +bhat[2,1]*x), sd=s, log = FALSE)*
             x^(ab[1,1]-1)*(1-x)^(ab[2,1]-1)}
fx<- function(x) {dnorm(yi, mean=(bhat[1,1]+bhat[2,1]*x), sd=s, log = FALSE)*
             x^ab[1,1]*(1-x)^(ab[2,1]-1)}
fx2<- function(x) {dnorm(yi, mean=(bhat[1,1]+bhat[2,1]*x), sd=s, log = FALSE)*
             x^(ab[1,1]+1)*(1-x)^(ab[2,1]-1)}
flx<- function(x) {dnorm(yi, mean=(bhat[1,1]+bhat[2,1]*x), sd=s, log = FALSE)*
             log(x)*x^(ab[1,1]-1)*(1-x)^(ab[2,1]-1)}
flx2<- function(x){dnorm(yi, mean=(bhat[1,1]+bhat[2,1]*x), sd=s, log = FALSE)*
          log(1-x)*x^(ab[1,1]-1)*(1-x)^(ab[2,1]-1)}

denom <-  integrate(fd, 0, 1)
numx  <-  integrate(fx, 0,1)
numx2 <-  integrate(fx2,0,1)
numlx <-  integrate(flx, 0,1)
numlx2<-  integrate(flx2,0,1)

xx[i,1] <-   numx$value/denom$value
xx[i,2] <-  numx2$value/denom$value
xx[i,3] <-  numlx$value/denom$value
xx[i,4] <- numlx2$value/denom$value
}  # End E-loop

xprimex <- matrix(1,2,2)
xprimex[1,1] <- n
xprimex[1,2] <- sum(xx[,1])
xprimex[2,1] <- xprimex[1,2]
xprimex[2,2] <- sum(xx[,2])

# M-step: Update parameter estimates
x0 <- matrix(1,n,2)
x0[,2] <- xx[,1]

bhat <-  solve(xprimex)%*%t(x0)%*%y
#s<-sqrt((t(y)%*%y - 2*t(bhat)%*%t(x0)%*%y+t(bhat)%*%xprimex%*%bhat)/n)
s<-sqrt((t(y)%*%y -  t(bhat)%*%xprimex%*%bhat)/n)





# Implement the Newton-Ralphson algorithm to estimate the parameters
# of the beta distribution
# First, get initial values via the method of moments
xbar <- as.matrix(apply(xx,2,mean))
m1 <- xbar[1,1]
m2 <- xbar[2,1]
ab[2,1] <- m1*(1-m1)^2/(m2-m1^2) + m1 -1
ab[1,1] <- ab[2,1]*m1/(1-m1)


S <- matrix(0,2,1)       # score vector
Info <- matrix(0,2,2)    # Information matrix
eps <- 0.0000001
change <- 100
while (change > eps) 
{
abold <- ab 
S[1,1] <- n*(digamma(ab[1,1]+ab[2,1])-digamma(ab[1,1]))+sum(xx[,3])
S[2,1] <- n*(digamma(ab[1,1]+ab[2,1])-digamma(ab[2,1]))+sum(xx[,4])

Info[1,1] <- n*(trigamma(ab[1,1]+ab[2,1])-trigamma(ab[1,1]))
Info[2,2] <- n*(trigamma(ab[1,1]+ab[2,1])-trigamma(ab[2,1]))
Info[2,1] <- n*(trigamma(ab[1,1]+ab[2,1]))
Info[1,2] <- Info[2,1]
Info<- -Info

ab <- ab + solve(Info)%*%S
change <- t(ab-abold)%*%(ab-abold)
#cat("change", change, "\n")
}

kb <- gamma(ab[1,1]+ab[2,1])/(gamma(ab[1,1])*gamma(ab[2,1])) # beta normalization constants

loglike <- -n/2*log(2*pi)-.5*n*log(s^2) -
            .5/s^2*
             (t(y)%*%y - 2*t(bhat)%*%t(x0)%*%y+t(bhat)%*%xprimex%*%bhat)+
              n*log(kb) +
              (ab[1,1]-1)*sum(xx[,3]) + 
                 (ab[2,1]-1)*sum(xx[,4])

diff <- sum((ab-abold)^2) + sum((bhat-bhatold)^2) + (s - sold)^2

abold <- ab
bhatold <- bhat
sold <- s

cat("iteration ", nit, "\n")  
cat("loglike =", loglike, "\n")
cat("Parameter   Estimate", "\n")       
cat("beta0:       ", bhat[1,1], "\n")
cat("beta1:       ", bhat[2,1], "\n")
cat("a:           ", ab[1,1], "\n")
cat("b:           ", ab[2,1], "\n")
cat("sigma:       ", s, "\n")
cat("diff :       ", diff, "\n")
cat("\n")
cat("\n")


bpfit<-gamma(ab[1,1]+ab[2,1])/(gamma(ab[1,1])*gamma(ab[2,1]))*
                       t^(ab[1,1]-1)*(1-t)^(ab[2,1]-1)
plot(t,bpfit, type='l', xlim=c(0,1),ylim=c(0,9), xlab="x")
}



yd=as.matrix(seq(-20,30, by=.001))
fy2=NULL
for (ij in 1:length(yd))
{


int2 <- function(x){
dnorm(yd[ij,1], mean=(bhat[1,1]+bhat[2,1]*x),sd=s,log = FALSE)*
     gamma(ab[1,1]+ab[2,1])/(gamma(ab[1,1])*gamma(ab[2,1]))*
     x^(ab[1,1]-1)*(1-x)^(ab[2,1]-1)}
fy12 <- integrate(int2,0,1)$value
fy2 = rbind(fy2, fy12)
}

# generate finite mixture density
emr <- t(as.matrix(muvar))
fmix <- 1/sqrt(2*pi)*(emr[1,1]*exp(-.5*(yd-emr[3,1])^2/
     emr[5,1])/sqrt(emr[5,1])+
     emr[2,1]*exp(-.5*(yd-emr[4,1])^2/
     emr[6,1])/sqrt(emr[6,1]))

sd=as.numeric(sqrt(var(y)))
ybar=apply(y,2,mean)
fnor <- 1/sqrt(2*pi*sd^2)*exp(-.5*(yd-ybar)^2/sd^2)

s2 <- s^2 + 
      bhat[2,1]^2*ab[1,1]*ab[2,1]/((ab[1,1]+ab[2,1]+1)*(ab[1,1]+ab[2,1])^2)
ybar2=bhat[1,1]+bhat[2,1]*(ab[1,1]/(ab[1,1]+ab[2,1]))
s2 <- as.numeric(s2)
fnor2 <- 1/sqrt(2*pi*s2)*exp(-.5*(yd-ybar2)^2/s2)



par(mfrow=c(2,1))

dy=density(y,bw=1.94)
hist(y,8, freq=FALSE, xlim=c(-10,30), ylim=c(0,0.12),
        ylab="Density", xlab="Difference", 
    main="Baseline - (Week 1) HAM-D")
lines(dy, type='l', lwd=2 )
lines(yd, fy2, lty=2, lwd=2, col=2)   # Latent Regression Fit
lines(yd, fmix, lty=5, lwd=2, col=3)  # Finite mixture fit
#lines(yd, fnor, lty=4)  # Normal MLE fit
#lines(yd, fnor2, lty=5) # Latent Reg. normal fit
#abline(v=bhat[1,1])
#abline(v=bhat[2,1]+bhat[1,1])



bpfit<-gamma(ab[1,1]+ab[2,1])/(gamma(ab[1,1])*gamma(ab[2,1]))*
                       t^(ab[1,1]-1)*(1-t)^(ab[2,1]-1)
plot(t,bpfit, type='l', xlim=c(0,1),ylim=c(0,7),xlab="x",
        ylab="", main="Latent Beta Density", lwd=3)

