

#  procedure EMNORM
#  procedure EMNORM  

# input:   k       Number of Mixture components
#          y       N-vector of observed data
#          prior   k-vector of initial prior probabilities
#          mu      k by p matrix of initial means
#          var     k array of initial p by p covariance matrices
#          pool    indicator for equality of variances:
#                  pool<-0: unequal variances; pool<-1: equal variances

#  output  prior   final initial probabilities
#          mu      final means
#          var     final variances
#          loglik  value of (complete data) log-likelihood
#          nit     number of iterations of EM-algorithm
#         post     posterior probability estimates


emnorm <- function(k,y, prior, mu, var, pool)
{
N <- dim(y)[1]                                    # number of observations 
p <- dim(y)[2]                                    # dimension of data
eps <- .000000001                                 # convergence criterion 
change <- 1                                       # initial test value for convergence 
maxiter <- 2000                                     # maximum number of iterations 
nit <- 0                                          # initialize iteration counter 
#param <- c(prior, mu, var)                       # arrange all parameters in a vector 
post <- matrix(0, N, k)                           # open matrix for posterior probabilities
while (change > eps && nit <= maxiter)            # start iterations 
{
options(digits = 6)                               # format for the display of numerical results
muold <- mu                                       # store old parameter values 
varold <- var
priorold <- prior

# Evaluate component densities and the log-likelihood
f <- matrix(0,N,k)
i <- 0
for(i in 1:k-1)
{
i <- i+1
expo <- -0.5*diag(as.matrix(sweep(y,2,mu[i,]))%*%solve(var[,,i])%*%as.matrix(t(sweep(y,2,mu[i,]))))
w <- eigen(var[,,i])
f[,i]  <-  prior[i,1]*(2*pi)^(-p*.5)/sqrt(exp(sum(log(w$values))))*exp(expo)
}
loglik <- sum(log(as.matrix(apply(f,1,sum))))   # evaluate log-likelihood function



# Compute posterier probabilities for each component
i <- 0
for(i in 1:k-1)
{
i <- i+1
post[,i]  <-  f[,i]/as.matrix(apply(f,1,sum))
}
post <- as.matrix(post)


# comment next line out if you don't want a protocol of the algorithm 
#cat(format(c(nit,loglik), justify = "right"), fill = T)

prior <- as.matrix(apply(post, 2, mean))                # M-step: prior probabilities 
                                   
mu <- as.matrix(t(post)) %*%as.matrix(y)/N              # M-step: means
i <- 0
for(i in 1:k-1)
{
i <- i+1
mu[i,] <- mu[i,]/prior[i,1]
}


varcommon <- matrix(0,p,p)
i <- 0
for(i in 1:k-1)
{
i <- i+1
var[,,i] <- 1/(N*prior[i,])*t(y)%*%as.matrix(diag(post[,i]))%*%as.matrix(y)- as.matrix(mu[i,])%*%mu[i,]     # m-step: variances
varcommon <- varcommon+var[,,i]*prior[i,1]
}                            
if (pool ==1)
{
var <-  array(varcommon, c(p,p,k))                                          # M-step: common variance 
}
change1 <- max(abs(prior - priorold))           # test value for convergence
change2 <- max(abs(mu-muold))
change3 <- max(abs(var-varold))
change  <-  max(change1,change2, change3)

 
nit <- nit + 1                               # increase interation counter 
}
if (nit >= maxiter)
cat("algorithm did not converge in ")   # warning message if convergence 
maxiter                                     # is not reached 
" iterations"
results <- list("prior" = prior, "mu" = mu, "var " = var,
"loglik" = loglik, "nit" = nit, "post" = post)
results
}




